@@ -34,9 +34,11 @@
 
 #include "fm-tree-model.h"
 #include "fm-properties-window.h"
+#include <string.h>
+#include <eel/eel-alert-dialog.h>
 #include <eel/eel-glib-extensions.h>
+#include <eel/eel-gtk-extensions.h>
 #include <eel/eel-preferences.h>
-#include <eel/eel-string.h>
 #include <eel/eel-stock-dialogs.h>
 #include <eel/eel-vfs-extensions.h>
 #include <gtk/gtkmain.h>
@@ -60,8 +62,11 @@
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libgnomevfs/gnome-vfs-volume-monitor.h>
 #include <libnautilus-private/nautilus-clipboard-monitor.h>
+#include <libnautilus-private/nautilus-desktop-icon-file.h>
+#include <libnautilus-private/nautilus-debug-log.h>
 #include <libnautilus-private/nautilus-file-attributes.h>
 #include <libnautilus-private/nautilus-file-operations.h>
+#include <libnautilus-private/nautilus-file-utilities.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
 #include <libnautilus-private/nautilus-program-choosing.h>
 #include <libnautilus-private/nautilus-tree-view-drag-dest.h>
@@ -104,6 +109,7 @@ struct FMTreeViewDetails {
 	GtkWidget *popup_paste;
 	GtkWidget *popup_rename;
 	GtkWidget *popup_trash;
+	GtkWidget *popup_delete;
 	GtkWidget *popup_properties;
 	GtkWidget *popup_unmount_separator;
 	GtkWidget *popup_unmount;
@@ -118,6 +124,7 @@ typedef struct {
 } PrependURIParameters;
 
 static GdkAtom copied_files_atom;
+static gboolean show_delete_command_auto_value;
 
 enum {
 	GNOME_COPIED_FILES
@@ -340,15 +347,21 @@ got_activation_uri_callback (NautilusFile *file, gpointer callback_data)
 
 	uri = nautilus_file_get_activation_uri (file);
 	if (uri != NULL
-	    && eel_str_has_prefix (uri, NAUTILUS_COMMAND_SPECIFIER)) {
+	    && g_str_has_prefix (uri, NAUTILUS_COMMAND_SPECIFIER)) {
 
 		uri += strlen (NAUTILUS_COMMAND_SPECIFIER);
+		nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
+				    "tree view launch_application_from_command window=%p: %s",
+				    view->details->window, uri);
 		nautilus_launch_application_from_command (screen, NULL, uri, NULL, FALSE);
 
 	} else if (uri != NULL
-	    	   && eel_str_has_prefix (uri, NAUTILUS_DESKTOP_COMMAND_SPECIFIER)) {
-		   
+		   && g_str_has_prefix (uri, NAUTILUS_DESKTOP_COMMAND_SPECIFIER)) {
+
 		file_uri = nautilus_file_get_uri (file);
+		nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
+				    "tree view launch_desktop_file window=%p: %s",
+				    view->details->window, file_uri);
 		nautilus_launch_desktop_file (screen, file_uri, NULL, NULL);
 		g_free (file_uri);
 		
@@ -361,6 +374,9 @@ got_activation_uri_callback (NautilusFile *file, gpointer callback_data)
 
 		/* Non-local executables don't get launched. They act like non-executables. */
 		if (file_uri == NULL) {
+			nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
+					    "tree view window_info_open_location window=%p: %s",
+					    view->details->window, uri);
 			nautilus_window_info_open_location
 				(view->details->window, 
 				 uri, 
@@ -368,6 +384,9 @@ got_activation_uri_callback (NautilusFile *file, gpointer callback_data)
 				 0,
 				 NULL);
 		} else {
+			nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
+					    "tree view launch_application_from_command window=%p: %s",
+					    view->details->window, file_uri);
 			nautilus_launch_application_from_command (screen, NULL, file_uri, NULL, FALSE);
 			g_free (file_uri);
 		}
@@ -379,6 +398,9 @@ got_activation_uri_callback (NautilusFile *file, gpointer callback_data)
 				g_free (view->details->selection_location);
 			}
 			view->details->selection_location = g_strdup (uri);
+			nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
+					    "tree view window_info_open_location window=%p: %s",
+					    view->details->window, uri);
 			nautilus_window_info_open_location
 				(view->details->window, 
 				 uri,
@@ -697,12 +719,36 @@ eject_for_type (GnomeVFSDeviceType type)
 	}
 }
 
+static gboolean
+is_parent_writable (NautilusFile *file)
+{
+	NautilusFile *parent;
+	gboolean result;
+	
+	parent = nautilus_file_get_parent (file);
+	
+	/* No parent directory, return FALSE */
+	if (parent == NULL) {
+		return FALSE;
+	}
+	
+	result = nautilus_file_can_write (parent);
+	nautilus_file_unref (parent);
+	
+	return result;	
+}
+
 static gboolean
 button_pressed_callback (GtkTreeView *treeview, GdkEventButton *event,
 			 FMTreeView *view)
 {
 	GtkTreePath *path, *cursor_path;
 	char *uri;
+	gboolean parent_file_is_writable;
+	gboolean file_is_home_or_desktop;
+	gboolean file_is_special_link;
+	gboolean can_move_file_to_trash;
+	gboolean can_delete_file;
 
 	if (event->button == 3) {
 		gboolean unmount_is_eject = FALSE;
@@ -736,9 +782,26 @@ button_pressed_callback (GtkTreeView *treeview, GdkEventButton *event,
 							copied_files_atom,
 							clipboard_contents_received_callback, view);
 		}
-		gtk_widget_set_sensitive (view->details->popup_trash, can_move_uri_to_trash (uri));
+		can_move_file_to_trash = can_move_uri_to_trash (uri);
+		gtk_widget_set_sensitive (view->details->popup_trash, can_move_file_to_trash);
 		g_free (uri);
 		
+		if (show_delete_command_auto_value) {
+			parent_file_is_writable = is_parent_writable (view->details->popup_file);
+			file_is_home_or_desktop = nautilus_file_is_home (view->details->popup_file)
+				|| nautilus_file_is_desktop_directory (view->details->popup_file);
+			file_is_special_link = NAUTILUS_IS_DESKTOP_ICON_FILE (view->details->popup_file);
+			
+			can_delete_file = parent_file_is_writable 
+				&& !file_is_home_or_desktop
+				&& !file_is_special_link;
+
+			gtk_widget_show (view->details->popup_delete);
+			gtk_widget_set_sensitive (view->details->popup_delete, can_delete_file);
+		} else {
+			gtk_widget_hide (view->details->popup_delete);
+		}
+		
 		volume = fm_tree_model_get_volume_for_root_node_file (view->details->child_model, view->details->popup_file);
 		if (volume) {
 			show_unmount = TRUE;
@@ -1035,6 +1098,92 @@ fm_tree_view_trash_cb (GtkWidget *menu_item,
 	g_free (directory_uri);
 }
 
+static GtkWindow *
+fm_tree_view_get_containing_window (FMTreeView *view)
+{
+	GtkWidget *window;
+
+	g_assert (FM_IS_TREE_VIEW (view));
+
+	window = gtk_widget_get_ancestor (GTK_WIDGET (view), GTK_TYPE_WINDOW);
+	if (window == NULL) {
+		return NULL;
+	}
+
+	return GTK_WINDOW (window);
+}
+
+static char *
+file_name_from_uri (const char *uri)
+{
+	NautilusFile *file;
+	char *file_name;
+	
+	file = nautilus_file_get (uri);
+	file_name = nautilus_file_get_display_name (file);
+	nautilus_file_unref (file);
+
+	return file_name;	
+}
+
+static gboolean
+confirm_delete_directly (FMTreeView *view,
+			 const char *directory_uri)
+{
+	GtkDialog *dialog;
+	char *file_name;
+	char *prompt;
+	int response;
+
+	file_name = file_name_from_uri (directory_uri);
+	
+	prompt = g_strdup_printf (_("Are you sure you want to permanently delete \"%s\"?"), 
+				  file_name);
+	g_free (file_name);
+
+	dialog = GTK_DIALOG (eel_alert_dialog_new (fm_tree_view_get_containing_window (view),
+				                   0,
+						   GTK_MESSAGE_WARNING,
+						   GTK_BUTTONS_NONE,
+						   prompt,
+						   _("If you delete an item, it is permanently lost.")));
+
+	gtk_dialog_add_button (dialog, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
+	gtk_dialog_add_button (dialog, GTK_STOCK_DELETE, GTK_RESPONSE_YES);
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_YES);
+	
+	g_free (prompt);
+	
+	response = gtk_dialog_run (dialog);
+	
+	gtk_object_destroy (GTK_OBJECT(dialog));	
+	
+	return (response == GTK_RESPONSE_YES);
+}
+
+static void
+fm_tree_view_delete_cb (GtkWidget *menu_item,
+		        FMTreeView *view)
+{
+	GList *uri_list;
+	char *directory_uri;
+		
+	if (!show_delete_command_auto_value) {
+		return;
+	}
+	
+	directory_uri = nautilus_file_get_uri (view->details->popup_file);
+	
+	if (confirm_delete_directly (view, directory_uri)) {
+		uri_list = NULL;
+		uri_list = g_list_prepend (uri_list, g_strdup (directory_uri));
+		
+		nautilus_file_operations_delete (uri_list, GTK_WIDGET (view));
+		eel_g_list_free_deep (uri_list);
+	}
+	g_free (directory_uri);
+}
+
 static void
 fm_tree_view_properties_cb (GtkWidget *menu_item,
 			    FMTreeView *view)
@@ -1092,7 +1241,7 @@ fm_tree_view_unmount_cb (GtkWidget *menu_item,
 static void
 create_popup_menu (FMTreeView *view)
 {
-	GtkWidget *popup, *menu_item, *menu_image, *separator_item;
+	GtkWidget *popup, *menu_item, *menu_image;
 	
 	popup = gtk_menu_new ();
 	
@@ -1119,10 +1268,8 @@ create_popup_menu (FMTreeView *view)
 	gtk_menu_shell_append (GTK_MENU_SHELL (popup), menu_item);
 	view->details->popup_open_in_new_window = menu_item;
 	
-	separator_item = gtk_separator_menu_item_new ();
-	gtk_widget_show (separator_item);
-	gtk_menu_shell_append (GTK_MENU_SHELL (popup), separator_item);
-	
+	eel_gtk_menu_append_separator (GTK_MENU (popup));
+
 	/* add the "create folder" menu item */
 	menu_item = gtk_image_menu_item_new_with_label (_("Create Folder"));
 	g_signal_connect (menu_item, "activate",
@@ -1132,9 +1279,7 @@ create_popup_menu (FMTreeView *view)
 	gtk_menu_shell_append (GTK_MENU_SHELL (popup), menu_item);
 	view->details->popup_create_folder = menu_item;
 	
-	separator_item = gtk_separator_menu_item_new ();
-	gtk_widget_show (separator_item);
-	gtk_menu_shell_append (GTK_MENU_SHELL (popup), separator_item);
+	eel_gtk_menu_append_separator (GTK_MENU (popup));
 	
 	/* add the "cut folder" menu item */
 	menu_item = gtk_image_menu_item_new_from_stock (GTK_STOCK_CUT, NULL);
@@ -1168,13 +1313,11 @@ create_popup_menu (FMTreeView *view)
 	gtk_menu_shell_append (GTK_MENU_SHELL (popup), menu_item);
 	view->details->popup_paste = menu_item;
 	
-	separator_item = gtk_separator_menu_item_new ();
-	gtk_widget_show (separator_item);
-	gtk_menu_shell_append (GTK_MENU_SHELL (popup), separator_item);
+	eel_gtk_menu_append_separator (GTK_MENU (popup));
 	
 	/* add the "move to trash" menu item */
-	menu_image = gtk_image_new_from_stock (GTK_STOCK_DELETE,
-					       GTK_ICON_SIZE_MENU);
+	menu_image = gtk_image_new_from_icon_name ("user-trash",
+						   GTK_ICON_SIZE_MENU);
 	gtk_widget_show (menu_image);
 	menu_item = gtk_image_menu_item_new_with_label (_("Move to Trash"));
 	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item),
@@ -1186,25 +1329,17 @@ create_popup_menu (FMTreeView *view)
 	gtk_menu_shell_append (GTK_MENU_SHELL (popup), menu_item);
 	view->details->popup_trash = menu_item;
 	
-	separator_item = gtk_separator_menu_item_new ();
-	gtk_widget_show (separator_item);
-	gtk_menu_shell_append (GTK_MENU_SHELL (popup), separator_item);
-	
-	/* add the "properties" menu item */
-	menu_item = gtk_image_menu_item_new_from_stock (GTK_STOCK_PROPERTIES, NULL);
+	/* add the "delete" menu item */
+	menu_item = gtk_menu_item_new_with_mnemonic (_("_Delete"));
 	g_signal_connect (menu_item, "activate",
-			  G_CALLBACK (fm_tree_view_properties_cb),
+			  G_CALLBACK (fm_tree_view_delete_cb),
 			  view);
 	gtk_widget_show (menu_item);
 	gtk_menu_shell_append (GTK_MENU_SHELL (popup), menu_item);
-	view->details->popup_properties = menu_item;
-
-	/* add the unmount separator menu item */
-	menu_item = gtk_separator_menu_item_new ();
-	gtk_widget_show (menu_item);
-	gtk_menu_shell_append (GTK_MENU_SHELL (popup), menu_item);
-	view->details->popup_unmount_separator = menu_item;
+	view->details->popup_delete = menu_item;
 	
+	eel_gtk_menu_append_separator (GTK_MENU (popup));
+
 	/* add the "Unmount" menu item */
 	menu_item = gtk_image_menu_item_new_with_label ("eject label");
 	g_signal_connect (menu_item, "activate",
@@ -1214,6 +1349,19 @@ create_popup_menu (FMTreeView *view)
 	gtk_menu_shell_append (GTK_MENU_SHELL (popup), menu_item);
 	view->details->popup_unmount = menu_item;
 
+	/* add the unmount separator menu item */
+	view->details->popup_unmount_separator =
+		GTK_WIDGET (eel_gtk_menu_append_separator (GTK_MENU (popup)));
+
+	/* add the "properties" menu item */
+	menu_item = gtk_image_menu_item_new_from_stock (GTK_STOCK_PROPERTIES, NULL);
+	g_signal_connect (menu_item, "activate",
+			  G_CALLBACK (fm_tree_view_properties_cb),
+			  view);
+	gtk_widget_show (menu_item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (popup), menu_item);
+	view->details->popup_properties = menu_item;
+
 	view->details->popup = popup;
 }
 
@@ -1241,7 +1389,7 @@ create_tree (FMTreeView *view)
 		(view->details->child_model, "row_loaded",
 		 G_CALLBACK (row_loaded_callback),
 		 view, G_CONNECT_AFTER);
-	home_uri = gnome_vfs_get_uri_from_local_path (g_get_home_dir ());
+	home_uri = nautilus_get_home_directory_uri ();
 	fm_tree_model_add_root_uri (view->details->child_model, home_uri, _("Home Folder"), "gnome-fs-home", NULL);
 	g_free (home_uri);
 	fm_tree_model_add_root_uri (view->details->child_model, "file:///", _("File System"), "gnome-fs-directory", NULL);
@@ -1251,6 +1399,7 @@ create_tree (FMTreeView *view)
 	
 	volume_monitor = gnome_vfs_get_volume_monitor ();
 	volumes = gnome_vfs_volume_monitor_get_mounted_volumes (volume_monitor);
+	volumes = g_list_sort (volumes, (GCompareFunc) gnome_vfs_volume_compare);
 	for (l = volumes; l != NULL; l = l->next) {
 		add_root_for_volume (view, l->data);
 		gnome_vfs_volume_unref (l->data);
@@ -1473,6 +1622,9 @@ fm_tree_view_class_init (FMTreeViewClass *class)
 	G_OBJECT_CLASS (class)->finalize = fm_tree_view_finalize;
 	
 	copied_files_atom = gdk_atom_intern ("x-special/gnome-copied-files", FALSE);
+
+	eel_preferences_add_auto_boolean (NAUTILUS_PREFERENCES_ENABLE_DELETE,
+					  &show_delete_command_auto_value);	
 }
 
 static const char *
