@@ -38,9 +38,11 @@
 #define NAUTILUS_ACTION_SHOW_HIDE_STATUSBAR "Show Hide Statusbar"
 #define NAUTILUS_ACTION_SHOW_HIDE_LOCATION_BAR "Show Hide Location Bar"
 #define NAUTILUS_ACTION_GO_TO_BURN_CD "Go to Burn CD"
+#define NAUTILUS_ACTION_HOME "Home"
 #define NAUTILUS_ACTION_ZOOM_IN "Zoom In"
 #define NAUTILUS_ACTION_ZOOM_OUT "Zoom Out"
 #define NAUTILUS_ACTION_ZOOM_NORMAL "Zoom Normal"
 #define NAUTILUS_ACTION_CLOSE "Close"
+#define NAUTILUS_ACTION_SEARCH "Search"
 
 #endif /* NAUTILUS_ACTIONS_H */
