@@ -37,6 +37,7 @@
 #include <bonobo-activation/bonobo-activation.h>
 #include <bonobo/bonobo-main.h>
 #include <dlfcn.h>
+#include <signal.h>
 #include <eel/eel-debug.h>
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-self-checks.h>
@@ -48,7 +49,9 @@
 #include <glib/gi18n.h>
 #include <libgnome/gnome-init.h>
 #include <libgnomeui/gnome-ui-init.h>
+#include <libgnomeui/gnome-client.h>
 #include <libgnomevfs/gnome-vfs-init.h>
+#include <libnautilus-private/nautilus-debug-log.h>
 #include <libnautilus-private/nautilus-directory-metafile.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
 #include <libnautilus-private/nautilus-lib-self-check-functions.h>
@@ -108,6 +111,15 @@ event_loop_unregister (GtkObject *object)
 	}
 }
 
+static gboolean
+initial_event_loop_needed (gpointer data)
+{
+	if (!is_event_loop_needed ()) {
+		eel_gtk_main_quit_all ();
+	}
+	return FALSE;
+}
+
 void
 nautilus_main_event_loop_register (GtkObject *object)
 {
@@ -123,8 +135,13 @@ nautilus_main_is_event_loop_mainstay (GtkObject *object)
 }
 
 void
-nautilus_main_event_loop_quit (void)
+nautilus_main_event_loop_quit (gboolean explicit)
 {
+	if (explicit) {
+		/* Explicit --quit, make sure we don't restart */
+		gnome_client_set_restart_style (gnome_master_client (),
+						GNOME_RESTART_IF_RUNNING);
+	}
 	while (event_loop_registrants != NULL) {
 		gtk_object_destroy (event_loop_registrants->data);
 	}
@@ -183,6 +200,157 @@ slowly_and_stupidly_obtain_timestamp (Display *xdisplay)
 	return event.xproperty.time;
 }
 
+static void
+dump_debug_log (void)
+{
+	char *filename;
+
+	filename = g_build_filename (g_get_home_dir (), "nautilus-debug-log.txt", NULL);
+	nautilus_debug_log_dump (filename, NULL); /* NULL GError */
+	g_free (filename);
+}
+
+static int debug_log_pipes[2];
+
+static gboolean
+debug_log_io_cb (GIOChannel *io, GIOCondition condition, gpointer data)
+{
+	char a;
+
+	while (read (debug_log_pipes[0], &a, 1) != 1)
+		;
+
+	nautilus_debug_log (TRUE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
+			    "user requested dump of debug log");
+
+	dump_debug_log ();
+	return FALSE;
+}
+
+/* sigaction structures for the old handlers of these signals */
+static struct sigaction old_segv_sa;
+static struct sigaction old_abrt_sa;
+static struct sigaction old_trap_sa;
+static struct sigaction old_fpe_sa;
+static struct sigaction old_bus_sa;
+
+static void
+sigusr1_handler (int sig)
+{
+	while (write (debug_log_pipes[1], "a", 1) != 1)
+		;
+}
+
+static void
+sigfatal_handler (int sig)
+{
+	void (* func) (int);
+
+	/* FIXME: is this totally busted?  We do malloc() inside these functions,
+	 * and yet we are inside a signal handler...
+	 */
+	nautilus_debug_log (TRUE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
+			    "debug log dumped due to signal %d", sig);
+	dump_debug_log ();
+
+	switch (sig) {
+	case SIGSEGV:
+		func = old_segv_sa.sa_handler;
+		break;
+
+	case SIGABRT:
+		func = old_abrt_sa.sa_handler;
+		break;
+
+	case SIGTRAP:
+		func = old_trap_sa.sa_handler;
+		break;
+
+	case SIGFPE:
+		func = old_fpe_sa.sa_handler;
+		break;
+
+	case SIGBUS:
+		func = old_bus_sa.sa_handler;
+		break;
+
+	default:
+		func = NULL;
+		break;
+	}
+
+	/* this scares me */
+	if (func != NULL && func != SIG_IGN && func != SIG_DFL)
+		(* func) (sig);
+}
+
+static void
+setup_debug_log_signals (void)
+{
+	struct sigaction sa;
+	GIOChannel *io;
+
+	if (pipe (debug_log_pipes) == -1)
+		g_error ("Could not create pipe() for debug log");
+
+	io = g_io_channel_unix_new (debug_log_pipes[0]);
+	g_io_add_watch (io, G_IO_IN, debug_log_io_cb, NULL);
+
+	sa.sa_handler = sigusr1_handler;
+	sigemptyset (&sa.sa_mask);
+	sa.sa_flags = 0;
+	sigaction (SIGUSR1, &sa, NULL);
+
+	sa.sa_handler = sigfatal_handler;
+	sigemptyset (&sa.sa_mask);
+	sa.sa_flags = 0;
+
+	sigaction(SIGSEGV, &sa, &old_segv_sa);
+	sigaction(SIGABRT, &sa, &old_abrt_sa);
+	sigaction(SIGTRAP, &sa, &old_trap_sa);
+	sigaction(SIGFPE,  &sa, &old_fpe_sa);
+	sigaction(SIGBUS,  &sa, &old_bus_sa);
+}
+
+static GLogFunc default_log_handler;
+
+static void
+log_override_cb (const gchar   *log_domain,
+		 GLogLevelFlags log_level,
+		 const gchar   *message,
+		 gpointer       user_data)
+{
+	gboolean is_debug;
+	gboolean is_milestone;
+
+	is_debug = ((log_level & G_LOG_LEVEL_DEBUG) != 0);
+	is_milestone = !is_debug;
+
+	nautilus_debug_log (is_milestone, NAUTILUS_DEBUG_LOG_DOMAIN_GLOG, "%s", message);
+
+	if (!is_debug)
+		(* default_log_handler) (log_domain, log_level, message, user_data);
+}
+
+static void
+setup_debug_log_glog (void)
+{
+	default_log_handler = g_log_set_default_handler (log_override_cb, NULL);
+}
+
+static void
+setup_debug_log (void)
+{
+	char *config_filename;
+
+	config_filename = g_build_filename (g_get_home_dir (), "nautilus-debug-log.conf", NULL);
+	nautilus_debug_log_load_configuration (config_filename, NULL); /* NULL GError */
+	g_free (config_filename);
+
+	setup_debug_log_signals ();
+	setup_debug_log_glog ();
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -272,6 +440,12 @@ main (int argc, char *argv[])
 				      GNOME_PARAM_HUMAN_READABLE_NAME, _("Nautilus"),
 				      NULL);
 
+	/* We do this after gnome_program_init(), since that function sets up
+	 * its own handler for SIGSEGV and others --- we want to chain to those
+	 * handlers.
+	 */
+	setup_debug_log ();
+
 	if (session_to_load != NULL) {
 		no_default_window = TRUE;
 	}
@@ -367,11 +541,18 @@ main (int argc, char *argv[])
 			 remaining);
 		g_free (startup_id_copy);
 
-		if (is_event_loop_needed ()) {
-			gtk_main ();
-		}
+		/* The application startup does things in an idle, so
+		   we need to check whether the main loop is needed in an idle
+		*/
+		g_idle_add (initial_event_loop_needed, NULL);
+		gtk_main ();
 	}
 
+	/* This has to be done before gnome_vfs_shutdown, because
+ 	 * it might call nautilus_file_get_uri() which might call
+ 	 * gnome_vfs_uri_append_string()
+ 	 */
+ 	eel_debug_shut_down ();
 
 	gnome_vfs_shutdown ();
 
@@ -382,8 +563,6 @@ main (int argc, char *argv[])
 		bonobo_object_unref (application);
 	}
 
-	eel_debug_shut_down ();
-
 	/* If told to restart, exec() myself again. This is used when
 	 * the program is told to restart with CORBA, for example when
 	 * an update takes place.
