@@ -49,7 +49,6 @@
 #include <eel/eel-gtk-macros.h>
 #include <eel/eel-stock-dialogs.h>
 #include <eel/eel-string.h>
-#include <eel/eel-vfs-extensions.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtkmain.h>
@@ -63,12 +62,7 @@
 #include <gtk/gtkliststore.h>
 #include <glib/gi18n.h>
 #include <libgnome/gnome-macros.h>
-#include <libgnome/gnome-util.h>
-#include <libgnomeui/gnome-messagebox.h>
 #include <libgnomeui/gnome-uidefs.h>
-#include <libgnomeui/gnome-window-icon.h>
-#include <libgnomevfs/gnome-vfs-uri.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 #include <libnautilus-private/nautilus-file-utilities.h>
 #include <libnautilus-private/nautilus-file-attributes.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
@@ -139,6 +133,48 @@ static void nautilus_navigation_window_show_location_bar_temporarily (NautilusNa
 GNOME_CLASS_BOILERPLATE (NautilusNavigationWindow, nautilus_navigation_window,
 			 NautilusWindow, NAUTILUS_TYPE_WINDOW)
 
+static void
+location_button_toggled_cb (GtkToggleButton *toggle,
+			    NautilusNavigationWindow *window)
+{
+	gboolean is_active;
+
+	is_active = gtk_toggle_button_get_active (toggle);
+	eel_preferences_set_boolean (NAUTILUS_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY, is_active);
+
+	if (is_active) {
+		nautilus_navigation_bar_activate (NAUTILUS_NAVIGATION_BAR (window->navigation_bar));
+	}
+}
+
+static gboolean
+location_button_should_be_active (NautilusNavigationWindow *window)
+{
+	return eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY);
+}
+
+static GtkWidget *
+location_button_create (NautilusNavigationWindow *window)
+{
+	GtkWidget *image;
+	GtkWidget *button;
+
+	image = gtk_image_new_from_stock (GTK_STOCK_EDIT, GTK_ICON_SIZE_BUTTON);
+	gtk_widget_show (image);
+
+	button = g_object_new (GTK_TYPE_TOGGLE_BUTTON,
+			       "image", image,
+			       "active", location_button_should_be_active (window),
+			       NULL);
+
+	gtk_tooltips_set_tip (window->details->tooltips, button,
+			      _("Toggle between button and text-based location bar"), NULL);
+
+	g_signal_connect (button, "toggled",
+			  G_CALLBACK (location_button_toggled_cb), window);
+	return button;
+}
+
 static void
 nautilus_navigation_window_instance_init (NautilusNavigationWindow *window)
 {
@@ -151,6 +187,10 @@ nautilus_navigation_window_instance_init (NautilusNavigationWindow *window)
 	
 	window->details = g_new0 (NautilusNavigationWindowDetails, 1);
 
+	window->details->tooltips = gtk_tooltips_new ();
+	g_object_ref (window->details->tooltips);
+	gtk_object_sink (GTK_OBJECT (window->details->tooltips));
+
 	window->details->content_paned = nautilus_horizontal_splitter_new ();
 	gtk_table_attach (GTK_TABLE (NAUTILUS_WINDOW (window)->details->table),
 			  window->details->content_paned,
@@ -206,7 +246,7 @@ nautilus_navigation_window_instance_init (NautilusNavigationWindow *window)
 	location_bar = gtk_toolbar_new ();
 	window->details->location_bar = location_bar;
 
-	hbox = gtk_hbox_new (FALSE, 0);
+	hbox = gtk_hbox_new (FALSE, 12);
 	gtk_widget_show (hbox);
 
 	item = gtk_tool_item_new ();
@@ -217,6 +257,10 @@ nautilus_navigation_window_instance_init (NautilusNavigationWindow *window)
 	gtk_toolbar_insert (GTK_TOOLBAR (location_bar),
 			    item, -1);
 
+	window->details->location_button = location_button_create (window);
+	gtk_box_pack_start (GTK_BOX (hbox), window->details->location_button, FALSE, FALSE, 0);
+	gtk_widget_show (window->details->location_button);
+
 	window->path_bar = g_object_new (NAUTILUS_TYPE_PATH_BAR, NULL);
  	gtk_widget_show (window->path_bar);
 	
@@ -305,14 +349,25 @@ static void
 always_use_location_entry_changed (gpointer callback_data)
 {
 	NautilusNavigationWindow *window;
+	gboolean use_entry;
 
 	window = NAUTILUS_NAVIGATION_WINDOW (callback_data);
 
-	if (eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY)) {
+	use_entry = eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY);
+
+	if (use_entry) {
 		nautilus_navigation_window_set_bar_mode (window, NAUTILUS_BAR_NAVIGATION);
 	} else {
 		nautilus_navigation_window_set_bar_mode (window, NAUTILUS_BAR_PATH);
 	}
+
+	g_signal_handlers_block_by_func (window->details->location_button,
+					 G_CALLBACK (location_button_toggled_cb),
+					 window);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (window->details->location_button), use_entry);
+	g_signal_handlers_unblock_by_func (window->details->location_button,
+					   G_CALLBACK (location_button_toggled_cb),
+					   window);
 }
 
 static int
@@ -593,6 +648,11 @@ nautilus_navigation_window_destroy (GtkObject *object)
 	window->zoom_control = NULL;
 
 	window->details->content_paned = NULL;
+
+	if (window->details->tooltips) {
+		g_object_unref (window->details->tooltips);
+		window->details->tooltips = NULL;
+	}
 	
 	GTK_OBJECT_CLASS (parent_class)->destroy (object);
 }
@@ -698,18 +758,6 @@ nautilus_navigation_window_go_forward (NautilusNavigationWindow *window)
 	nautilus_navigation_window_back_or_forward (window, FALSE, 0);
 }
 
-void
-nautilus_navigation_window_go_home (NautilusNavigationWindow *window)
-{
-	char *home_uri;
-
-	home_uri = gnome_vfs_get_uri_from_local_path (g_get_home_dir ());
-	
-	g_assert (home_uri != NULL);
-	nautilus_window_go_to (NAUTILUS_WINDOW (window), home_uri);
-	g_free (home_uri);
-}
-
 void
 nautilus_navigation_window_allow_back (NautilusNavigationWindow *window, gboolean allow)
 {
@@ -973,7 +1021,7 @@ static void
 search_bar_activate_callback (NautilusSearchBar *bar,
 			      NautilusWindow *window)
 {
-	char *uri, *home_uri;
+	char *uri, *current_uri;
 	NautilusDirectory *directory;
 	NautilusSearchDirectory *search_directory;
 	NautilusQuery *query;
@@ -988,9 +1036,9 @@ search_bar_activate_callback (NautilusSearchBar *bar,
 	query = nautilus_search_bar_get_query (NAUTILUS_SEARCH_BAR (NAUTILUS_NAVIGATION_WINDOW (window)->search_bar));
 	if (query != NULL) {
 		if (!nautilus_search_directory_is_indexed (search_directory)) {
-			home_uri = nautilus_get_home_directory_uri ();
-			nautilus_query_set_location (query, home_uri);
-			g_free (home_uri);
+			current_uri = nautilus_window_get_location (window);
+			nautilus_query_set_location (query, current_uri);
+			g_free (current_uri);
 		}
 		nautilus_search_directory_set_query (search_directory, query);
 		g_object_unref (query);
@@ -1134,7 +1182,6 @@ add_sidebar_panels (NautilusNavigationWindow *window)
 	GList *providers;
 	GList *p;
 	NautilusSidebar *sidebar_panel;
-	NautilusSidebarProvider *provider;
 
 	g_assert (NAUTILUS_IS_NAVIGATION_WINDOW (window));
 
@@ -1145,6 +1192,8 @@ add_sidebar_panels (NautilusNavigationWindow *window)
  	providers = nautilus_module_get_extensions_for_type (NAUTILUS_TYPE_SIDEBAR_PROVIDER);
 	
 	for (p = providers; p != NULL; p = p->next) {
+		NautilusSidebarProvider *provider;
+
 		provider = NAUTILUS_SIDEBAR_PROVIDER (p->data);
 		
 		sidebar_panel = nautilus_sidebar_provider_create (provider,
@@ -1154,7 +1203,9 @@ add_sidebar_panels (NautilusNavigationWindow *window)
 		
 		g_object_unref (sidebar_panel);
 	}
-	
+
+	nautilus_module_extension_list_free (providers);
+
 	current = nautilus_side_pane_get_current_panel (window->sidebar);
 	set_current_side_panel
 		(window, 
