@@ -36,37 +36,24 @@
 #include "nautilus-bookmarks-window.h"
 #include "nautilus-information-panel.h"
 #include "nautilus-main.h"
-#include "nautilus-signaller.h"
 #include "nautilus-window-manage-views.h"
 #include "nautilus-window-bookmarks.h"
 #include "nautilus-zoom-control.h"
 #include "nautilus-search-bar.h"
 #include <eel/eel-debug.h>
 #include <eel/eel-marshal.h>
-#include <eel/eel-gdk-extensions.h>
-#include <eel/eel-gdk-pixbuf-extensions.h>
-#include <eel/eel-gtk-extensions.h>
 #include <eel/eel-gtk-macros.h>
-#include <eel/eel-stock-dialogs.h>
 #include <eel/eel-string.h>
-#include <eel/eel-vfs-extensions.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gdk/gdkx.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkmenubar.h>
 #include <gtk/gtkmenuitem.h>
-#include <gtk/gtkoptionmenu.h>
 #include <gtk/gtktogglebutton.h>
 #include <gtk/gtkvbox.h>
 #include <glib/gi18n.h>
-#include <libgnome/gnome-macros.h>
-#include <libgnome/gnome-util.h>
-#include <libgnomeui/gnome-messagebox.h>
-#include <libgnomeui/gnome-uidefs.h>
-#include <libgnomeui/gnome-window-icon.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 #include <libnautilus-private/nautilus-file-utilities.h>
 #include <libnautilus-private/nautilus-file-attributes.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
@@ -79,6 +66,7 @@
 #include <libnautilus-private/nautilus-clipboard.h>
 #include <libnautilus-private/nautilus-undo.h>
 #include <libnautilus-private/nautilus-search-directory.h>
+#include <libnautilus-private/nautilus-signaller.h>
 #include <math.h>
 #include <sys/time.h>
 
@@ -190,6 +178,10 @@ nautilus_window_init (NautilusWindow *window)
 				 G_CALLBACK (icons_changed_callback), window,
 				 0);
 
+	/* Register to menu provider extension signal managing menu updates */
+	g_signal_connect_object (nautilus_signaller_get_current (), "popup_menu_changed",
+			 G_CALLBACK (nautilus_window_load_extension_menus), window, G_CONNECT_SWAPPED);
+
 	gtk_quit_add_destroy (1, GTK_OBJECT (window));
 
 	/* Keep the main event loop alive as long as the window exists */
@@ -255,8 +247,6 @@ nautilus_window_go_up_signal (NautilusWindow *window, gboolean close_behind)
 	return TRUE;
 }
 
-
-
 void
 nautilus_window_go_up (NautilusWindow *window, gboolean close_behind)
 {
@@ -362,7 +352,6 @@ nautilus_window_allow_reload (NautilusWindow *window, gboolean allow)
 	action = gtk_action_group_get_action (window->details->main_action_group,
 					      NAUTILUS_ACTION_RELOAD);
 	gtk_action_set_sensitive (action, allow);
-
 }
 
 void
@@ -372,10 +361,8 @@ nautilus_window_go_home (NautilusWindow *window)
 
 	g_return_if_fail (NAUTILUS_IS_WINDOW (window));
 
-	home_uri = gnome_vfs_get_uri_from_local_path (g_get_home_dir ());
-	
-	g_assert (home_uri != NULL);
-	nautilus_window_go_to (window, home_uri);
+	home_uri = nautilus_get_home_directory_uri ();
+	nautilus_window_open_location (window, home_uri, FALSE);
 	g_free (home_uri);
 }
 
@@ -1352,7 +1339,7 @@ remove_from_history_list (NautilusBookmark *bookmark)
 	}
 }
 
-static void
+static gboolean
 add_to_history_list (NautilusBookmark *bookmark)
 {
 	/* Note that the history is shared amongst all windows so
@@ -1389,8 +1376,10 @@ add_to_history_list (NautilusBookmark *bookmark)
 			}
 		}
 
-		nautilus_send_history_list_changed ();
+		return TRUE;
 	}
+
+	return FALSE;
 }
 
 void
@@ -1403,12 +1392,30 @@ nautilus_remove_from_history_list_no_notify (const char *uri)
 	g_object_unref (bookmark);
 }
 
+gboolean
+nautilus_add_to_history_list_no_notify (const char *uri,
+					const char *name,
+					gboolean has_custom_name,
+					const char *icon)
+{
+	NautilusBookmark *bookmark;
+	gboolean ret;
+
+	bookmark = nautilus_bookmark_new_with_icon (uri, name, has_custom_name, icon);
+	ret = add_to_history_list (bookmark);
+	g_object_unref (bookmark);
+
+	return ret;
+}
+
 static void
 real_add_current_location_to_history_list (NautilusWindow *window)
 {
         g_assert (NAUTILUS_IS_WINDOW (window));
                 
-        add_to_history_list (window->current_location_bookmark);
+	if (add_to_history_list (window->current_location_bookmark)) {
+		nautilus_send_history_list_changed ();
+	}
 }
 
 void
